@@ -1,4 +1,4 @@
-/*  $Id: rnews.c,v 1.54 2003/02/02 23:40:05 rra Exp $
+/*  $Id: rnews.c 7424 2005-10-09 05:04:12Z eagle $
 **
 **  A front-end for InterNetNews.
 **
@@ -14,6 +14,7 @@
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <pwd.h>
 #include <syslog.h>
 #include <sys/stat.h>
 
@@ -158,13 +159,14 @@ static char *REMclean(char *buff)
 /*
 **  Write an article to the rejected directory.
 */
-static void Reject(const char *article, const char *reason, const char *arg)
+static void
+Reject(const char *article, size_t length UNUSED, const char *reason,
+       const char *arg)
 {
 #if	defined(DO_RNEWS_SAVE_BAD)
     char *filename;
     FILE *F;
     int fd;
-    size_t length;
 #endif	/* defined(DO_RNEWS_SAVE_BAD) */
 
     notice(reason, arg);
@@ -186,7 +188,6 @@ static void Reject(const char *article, const char *reason, const char *arg)
         warn("cannot fdopen %s", filename);
 	return;
     }
-    length = strlen(article);
     if (fwrite(article, 1, length, F) != length)
         warn("cannot fwrite to %s", filename);
     if (fclose(F) == EOF)
@@ -201,7 +202,8 @@ static void Reject(const char *article, const char *reason, const char *arg)
 **  whole batch needs to be saved (such as when the server goes down or if
 **  the file is corrupted).
 */
-static bool Process(char *article)
+static bool
+Process(char *article, size_t artlen)
 {
     HEADER	        *hp;
     const char	        *p;
@@ -222,15 +224,15 @@ static bool Process(char *article)
 	return true;
 
     /* Convert the article to wire format. */
-    wirefmt = ToWireFmt(article, strlen(article), &length);
+    wirefmt = ToWireFmt(article, artlen, &length);
 
     /* Make sure that all the headers are there, note the ID. */
     for (hp = RequiredHeaders; hp < ARRAY_END(RequiredHeaders); hp++) {
         p = wire_findheader(wirefmt, length, hp->Name);
         if (p == NULL) {
             free(wirefmt);
-	    Reject(article, "bad_article missing %s", hp->Name);
-	    return false;
+	    Reject(article, artlen, "bad_article missing %s", hp->Name);
+	    return true;
 	}
 	if (IS_MESGID(hp)) {
 	    id = p;
@@ -248,8 +250,9 @@ static bool Process(char *article)
     /* Send the NNTP "ihave" message. */
     if ((p = strchr(id, '\r')) == NULL) {
         free(wirefmt);
-	Reject(article, "bad_article unterminated %s header", "Message-ID");
-	return false;
+	Reject(article, artlen, "bad_article unterminated %s header",
+               "Message-ID");
+	return true;
     }
     msgid = xstrndup(id, p - id);
     fprintf(ToServer, "ihave %s\r\n", msgid);
@@ -261,7 +264,10 @@ static bool Process(char *article)
     /* Get a reply, see if they want the article. */
     if (fgets(buff, sizeof buff, FromServer) == NULL) {
         free(wirefmt);
-        syswarn("cannot fgets after ihave");
+        if (ferror(FromServer))
+            syswarn("cannot fgets after ihave");
+        else
+            warn("unexpected EOF from server after ihave");
 	return false;
     }
     REMclean(buff);
@@ -273,8 +279,8 @@ static bool Process(char *article)
     switch (atoi(buff)) {
     default:
         free(wirefmt);
-	Reject(article, "unknown_reply after ihave %s", buff);
-	return true;
+	notice("unknown_reply after ihave %s", buff);
+	return false;
     case NNTP_RESENDIT_VAL:
         free(wirefmt);
 	return false;
@@ -312,7 +318,10 @@ static bool Process(char *article)
 
     /* Process server reply code. */
     if (fgets(buff, sizeof buff, FromServer) == NULL) {
-        syswarn("cant fgets after article");
+        if (ferror(FromServer))
+            syswarn("cannot fgets after article");
+        else
+            warn("unexpected EOF from server after article");
 	return false;
     }
     REMclean(buff);
@@ -329,7 +338,7 @@ static bool Process(char *article)
     case NNTP_TOOKIT_VAL:
 	break;
     case NNTP_REJECTIT_VAL:
-	Reject(article, "rejected %s", buff);
+	Reject(article, artlen, "rejected %s", buff);
 	break;
     }
     return true;
@@ -391,7 +400,7 @@ ReadRemainder(int fd, char first, char second)
 	article[used++] = '\n';
     article[used] = '\0';
 
-    ok = Process(article);
+    ok = Process(article, used);
     free(article);
     return ok;
 }
@@ -434,11 +443,13 @@ ReadBytecount(int fd, int artsize)
 #endif	/* 0 */
 	    return true;
 	}
-    if (p[-1] != '\n')
+    if (p[-1] != '\n') {
 	*p++ = '\n';
+        artsize++;
+    }
     *p = '\0';
 
-    return Process(article);
+    return Process(article, artsize);
 }
 
 
@@ -614,9 +625,9 @@ Unspool(void)
     bool	ok;
     struct stat		Sb;
     char		hostname[10];
-    int			fd, fd2;
+    int			fd;
     size_t		i;
-    char                *badname, *uuhost;
+    char                *uuhost;
 
     message_handlers_die(2, message_log_stderr, message_log_syslog_err);
     message_handlers_warn(2, message_log_stderr, message_log_syslog_err);
@@ -667,21 +678,15 @@ Unspool(void)
 	WaitForChildren(i);
 	UUCPHost = uuhost;
 
-	if (!ok) {
-            badname = concat(PathBadNews, "/", hostname, "XXXXXX", (char *) 0);
-            fd2 = mkstemp(badname);
-            if (fd2 < 0)
-                sysdie("cannot create temporary file");
-            close(fd2);
-            warn("cant unspool saving to %s", badname);
-	    if (rename(InputFile, badname) < 0)
-                sysdie("cannot rename %s to %s", InputFile, badname);
-	    close(fd);
-	    continue;
-	}
+        /* If UnpackOne returned true, the article has been dealt with one way
+           or the other, so remove it.  Otherwise, leave it in place; either
+           we got an unknown error from the server or we got a deferral, and
+           for both we want to try later. */
+	if (ok) {
+            if (unlink(InputFile) < 0)
+                syswarn("cannot remove %s", InputFile);
+        }
 
-	if (unlink(InputFile) < 0)
-            syswarn("cannot remove %s", InputFile);
 	close(fd);
     }
     closedir(dp);
@@ -822,16 +827,51 @@ int main(int ac, char *av[])
     message_handlers_warn(1, message_log_syslog_err);
     message_handlers_die(1, message_log_syslog_err);
 
-    if (setgid(getegid()) < 0)
-        die("cannot setgid to %lu", (unsigned long) getegid());
-    if (setuid(geteuid()) < 0)
-        die("cannot setuid to %lu", (unsigned long) geteuid());
+    /* The reason for the following is somewhat obscure and is done only
+       because rnews is sometimes installed setuid.
+
+       The stderr stream used by message_log_syslog_err is associated with
+       file descriptor 2, generally even if that file descriptor is closed.
+       Someone running rnews may close all of the standard file descriptors
+       before running it, in which case, later in its operations, one of the
+       article files or network connections it has open could be file
+       descriptor 2.  If an error occurs at that point, the error message may
+       be written to that file or network connection instead of to stderr,
+       with unpredictable results.
+
+       We avoid this by burning three file descriptors if the real and
+       effective user IDs don't match, or if we're running as root.  (If they
+       do match, there is no escalation of privileges and at worst the user is
+       just managing to produce a strange bug.) */
+    if (getuid() != geteuid() || geteuid() == 0) {
+        if (open("/dev/null", O_RDONLY) < 0)
+            sysdie("cannot open /dev/null");
+        if (open("/dev/null", O_RDONLY) < 0)
+            sysdie("cannot open /dev/null");
+        if (open("/dev/null", O_RDONLY) < 0)
+            sysdie("cannot open /dev/null");
+    }
+
+    /* Make sure that we switch to the news user if we're running as root,
+       since we may spool files and don't want those files owned by root.
+       Don't require that we be running as the news user, though; there are
+       other setups where rnews might be setuid news or be run by other
+       processes in the news group. */
+    if (getuid() == 0 || geteuid() == 0) {
+        struct passwd *pwd;
+
+        pwd = getpwnam(NEWSUSER);
+        if (pwd == NULL)
+            die("can't resolve %s to a UID (account doesn't exist?)",
+                NEWSUSER);
+        setuid(pwd->pw_uid);
+    }
 
     if (!innconf_read(NULL))
         exit(1);
-     UUCPHost = getenv(_ENV_UUCPHOST);
-     PathBadNews = concatpath(innconf->pathincoming, _PATH_BADNEWS);
-     port = innconf->nnrpdpostport;
+    UUCPHost = getenv(_ENV_UUCPHOST);
+    PathBadNews = concatpath(innconf->pathincoming, _PATH_BADNEWS);
+    port = innconf->nnrpdpostport;
 
     umask(NEWSUMASK);
 
