@@ -1,4 +1,4 @@
-/*  $Id: cc.c,v 1.107.2.2 2003/12/29 05:02:53 rra Exp $
+/*  $Id: cc.c 7416 2005-10-09 04:29:52Z eagle $
 **
 **  Routines for the control channel.
 **
@@ -1800,7 +1800,12 @@ CCreader(CHANNEL *cp)
 
     memcpy (&protocol,buff,sizeof (protocol)) ;
     memcpy (&bufflen,buff + sizeof (protocol),sizeof (bufflen)) ;
-    bufflen = ntohs (bufflen) - HEADER_SIZE ;
+    bufflen = ntohs (bufflen);
+    if (bufflen < HEADER_SIZE) {
+	syslog(L_ERROR, "%s cant read CCreader bad length", LogName);
+	return;
+    }
+    bufflen -= HEADER_SIZE ;
     
     i = RECVorREAD(CCchan->fd, buff, bufflen) ;
 
@@ -1924,6 +1929,7 @@ CCsetup(void)
     int			i;
 #if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
     struct sockaddr_un	server;
+    int size = 65535;
 #endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
 
     if (CCpath == NULL)
@@ -1953,6 +1959,10 @@ CCsetup(void)
 	syslog(L_FATAL, "%s cant socket unbound %m", LogName);
 	exit(1);
     }
+
+    /* Increase the buffer size for the Unix domain socket */
+    if (setsockopt(CCwriter, SOL_SOCKET, SO_SNDBUF, &size, sizeof(size)) < 0)
+	syslog(L_ERROR, "%s cant setsockopt %m", LogName);
 #else
     /* Create a named pipe and open it. */
     if (mkfifo(CCpath, 0666) < 0) {
