@@ -1,4 +1,4 @@
-/*  $Id: nc.c,v 1.89 2003/01/18 22:26:47 rra Exp $
+/*  $Id: nc.c 7418 2005-10-09 04:37:05Z eagle $
 **
 **  Routines for the NNTP channel.  Other channels get the descriptors which
 **  we turn into NNTP channels, and over which we speak NNTP.
@@ -127,21 +127,22 @@ NCwritereply(CHANNEL *cp, const char *text)
     WCHANappend(cp, text, strlen(text));	/* text in buffer */
     WCHANappend(cp, NCterm, strlen(NCterm));	/* add CR NL to text */
 
-    /* FIXME: Something is wrong with this code.  At the least, it's
-       confusing.  bp->used can be incremented without decrementing bp->left,
-       which shouldn't be allowed. */
     if (i == 0) {	/* if only data then try to write directly */
 	i = write(cp->fd, &bp->data[bp->used], bp->left);
 	if (Tracing || cp->Tracing)
-	    syslog(L_TRACE, "%s NCwritereply %d=write(%d, \"%.15s\", %d)",
-		CHANname(cp), i, cp->fd, &bp->data[bp->used], bp->left);
-	if (i > 0) bp->used += i;
+	    syslog(L_TRACE, "%s NCwritereply %d=write(%d, \"%.15s\", %lu)",
+		CHANname(cp), i, cp->fd, &bp->data[bp->used],
+		(unsigned long) bp->left);
+	if (i > 0)
+            bp->used += i;
 	if (bp->used == bp->left) {
 	    /* all the data was written */
 	    bp->used = bp->left = 0;
 	    NCwritedone(cp);
-	}
-	else i = 0;
+	} else {
+            bp->left -= i;
+            i = 0;
+        }
     } else i = 0;
     if (i <= 0) {	/* write failed, queue it for later */
 	WCHANadd(cp);
@@ -255,6 +256,7 @@ NCwritedone(CHANNEL *cp)
     case CSgetheader:
     case CSgetbody:
     case CSgetxbatch:
+    case CSgotlargearticle:
     case CScancel:
 	RCHANadd(cp);
 	break;
@@ -734,7 +736,8 @@ NCproc(CHANNEL *cp)
 
   readmore = movedata = false;
   if (Tracing || cp->Tracing)
-    syslog(L_TRACE, "%s NCproc Used=%d", CHANname(cp), cp->In.used);
+    syslog(L_TRACE, "%s NCproc Used=%lu", CHANname(cp),
+           (unsigned long) cp->In.used);
 
   bp = &cp->In;
   if (bp->used == 0)
@@ -742,8 +745,9 @@ NCproc(CHANNEL *cp)
 
   for ( ; ; ) {
     if (Tracing || cp->Tracing) {
-      syslog(L_TRACE, "%s cp->Start=%d cp->Next=%d bp->Used=%d", CHANname(cp),
-	cp->Start, cp->Next, bp->used);
+      syslog(L_TRACE, "%s cp->Start=%lu cp->Next=%lu bp->Used=%lu",
+        CHANname(cp), (unsigned long) cp->Start, (unsigned long) cp->Next,
+        (unsigned long) bp->used);
       if (bp->used > 15)
 	syslog(L_TRACE, "%s NCproc state=%d next \"%.15s\"", CHANname(cp),
 	  cp->State, &bp->data[cp->Next]);
@@ -896,23 +900,9 @@ NCproc(CHANNEL *cp)
 	break;
       }
 
-      if (*cp->Error != '\0') {
-	cp->Rejected++;
-	cp->State = CSgetcmd;
-	cp->Start = cp->Next;
-	NCclearwip(cp);
-	if (cp->Sendid.size > 3)
-	  NCwritereply(cp, cp->Sendid.data);
-	else
-	  NCwritereply(cp, cp->Error);
-	readmore = false;
-	movedata = false;
-	break;
-      }
-
       if (cp->State == CSgotlargearticle) {
 	syslog(L_NOTICE, "%s internal rejecting huge article (%d > %ld)",
-	  CHANname(cp), cp->LargeArtSize, innconf->maxartsize);
+	  CHANname(cp), cp->Next - cp->Start, innconf->maxartsize);
 	if (cp->Sendid.size)
 	  NCwritereply(cp, cp->Sendid.data);
 	else {
@@ -921,7 +911,6 @@ NCproc(CHANNEL *cp)
                    NNTP_REJECTIT_VAL, innconf->maxartsize);
 	  NCwritereply(cp, buff);
         }
-	cp->LargeArtSize = 0;
 	cp->State = CSgetcmd;
 	cp->Rejected++;
 	cp->Start = cp->Next;
@@ -940,6 +929,20 @@ NCproc(CHANNEL *cp)
 	break;
       }
 
+      if (*cp->Error != '\0') {
+	cp->Rejected++;
+	cp->State = CSgetcmd;
+	cp->Start = cp->Next;
+	NCclearwip(cp);
+	if (cp->Sendid.size > 3)
+	  NCwritereply(cp, cp->Sendid.data);
+	else
+	  NCwritereply(cp, cp->Error);
+	readmore = false;
+	movedata = false;
+	break;
+      }
+
       if (cp->State == CSnoarticle) {
 	/* this should happen when parsing header */
 	cp->Rejected++;
@@ -1034,8 +1037,8 @@ NCproc(CHANNEL *cp)
        * directory with an unique timestamp, and start rnews on it.
        */
       if (Tracing || cp->Tracing)
-	syslog(L_TRACE, "%s CSgetxbatch: now %d of %d bytes", CHANname(cp),
-	  bp->used, cp->XBatchSize);
+	syslog(L_TRACE, "%s CSgetxbatch: now %lu of %d bytes", CHANname(cp),
+	  (unsigned long) bp->used, cp->XBatchSize);
 
       if (cp->Next != 0) {
 	/* data must start from the begining of the buffer */
@@ -1118,8 +1121,9 @@ NCproc(CHANNEL *cp)
     if (cp->State == CSwritegoodbye || cp->Type == CTfree)
       break;
     if (Tracing || cp->Tracing)
-      syslog(L_TRACE, "%s NCproc state=%d Start=%d Next=%d Used=%d",
-	CHANname(cp), cp->State, cp->Start, cp->Next, bp->used);
+      syslog(L_TRACE, "%s NCproc state=%d Start=%lu Next=%lu Used=%lu",
+	CHANname(cp), cp->State, (unsigned long) cp->Start,
+        (unsigned long) cp->Next, (unsigned long) bp->used);
 
     if (movedata) { /* move data rather than extend buffer */
       TMRstart(TMR_DATAMOVE);
@@ -1164,8 +1168,8 @@ NCreader(CHANNEL *cp)
     int			i;
 
     if (Tracing || cp->Tracing)
-	syslog(L_TRACE, "%s NCreader Used=%d",
-	    CHANname(cp), cp->In.used);
+	syslog(L_TRACE, "%s NCreader Used=%lu",
+	    CHANname(cp), (unsigned long) cp->In.used);
 
     /* Read any data that's there; ignore errors (retry next time it's our
      * turn) and if we got nothing, then it's EOF so mark it closed. */
