@@ -1,4 +1,4 @@
-/*  $Id: nnrpd.c,v 1.171 2003/03/10 01:40:49 rra Exp $
+/*  $Id: nnrpd.c 7269 2005-06-07 03:47:54Z eagle $
 **
 **  NNTP server for readers (NNRP) for InterNetNews.
 **
@@ -79,7 +79,6 @@ char	*HISTORY = NULL;
 char	*NEWSGROUPS = NULL;
 char	*NNRPACCESS = NULL;
 
-bool	ForceReadOnly = false;
 static char 	*LocalLogFileName = NULL;
 static char 	*LocalLogDirName;
 
@@ -105,8 +104,6 @@ bool   PerlLoaded = false;
 bool PY_use_dynamic = false;
 #endif /* DO_PYTHON */
 
-bool LLOGenable;
-
 static char	CMDfetchhelp[] = "[MessageID|Number]";
 
 static CMDENT	CMDtable[] = {
@@ -244,8 +241,10 @@ ExitWithStats(int x, bool readconf)
     if (History)
 	HISclose(History);
 
-    TMRsummary(ClientHost, timer_name);
-    TMRfree();
+    if (innconf->timer != 0) {
+        TMRsummary(ClientHost, timer_name);
+        TMRfree();
+    }
 
     if (LocalLogFileName != NULL)
 	free(LocalLogFileName);
@@ -656,11 +655,11 @@ void
 Printf(const char *fmt, ...)
 {
     va_list     args;
-    char        buff[2048];
 
 #ifdef HAVE_SSL
     if (tls_conn) {
       int r;
+      char buff[2048];
 
       va_start(args, fmt);
       vsnprintf(buff, sizeof(buff), fmt, args);
@@ -743,7 +742,6 @@ WaitChild(int s NO_SIGACTION_UNUSED)
 
 static void SetupDaemon(void) {
     bool                val;
-    time_t statinterval;
 
     val = true;
     if (SMsetup(SM_PREOPEN, (void *)&val) && !SMinit()) {
@@ -863,9 +861,9 @@ main(int argc, char *argv[])
         exit(1);
 
 #ifdef HAVE_SSL
-    while ((i = getopt(argc, argv, "c:b:Dfi:I:g:nop:P:Rr:s:tS")) != EOF)
+    while ((i = getopt(argc, argv, "c:b:Dfi:I:g:nop:P:r:s:tS")) != EOF)
 #else
-    while ((i = getopt(argc, argv, "c:b:Dfi:I:g:nop:P:Rr:s:t")) != EOF)
+    while ((i = getopt(argc, argv, "c:b:Dfi:I:g:nop:P:r:s:t")) != EOF)
 #endif /* HAVE_SSL */
 	switch (i) {
 	default:
@@ -907,9 +905,6 @@ main(int argc, char *argv[])
  	case 'p':			/* tcp port for daemon mode */
  	    ListenPort = atoi(optarg);
  	    break;
-	case 'R':			/* Ignore 'P' option in access file */
-	    ForceReadOnly = true;
-	    break;
 	case 'r':			/* Reject connection message */
 	    Reject = xstrdup(optarg);
 	    break;
@@ -1129,7 +1124,8 @@ main(int argc, char *argv[])
 	close(fd);
 	dup2(0, 1);
 	dup2(0, 2);
-        TMRinit(TMR_MAX);
+        if (innconf->timer != 0)
+            TMRinit(TMR_MAX);
         STATstart = TMRnow_double();
 	SetupDaemon();
 
@@ -1143,7 +1139,8 @@ main(int argc, char *argv[])
 	xsignal(SIGCHLD, SIG_DFL);
  
     } else {
-        TMRinit(TMR_MAX);
+        if (innconf->timer)
+            TMRinit(TMR_MAX);
         STATstart = TMRnow_double();
 	SetupDaemon();
 	/* Arrange to toggle tracing. */
