@@ -1,4 +1,4 @@
-/*  $Id: overdata.c,v 1.12.2.1 2003/05/31 19:48:27 rra Exp $
+/*  $Id: overdata.c 7477 2005-12-24 21:34:38Z eagle $
 **
 **  Overview data processing.
 **
@@ -56,7 +56,8 @@ overview_fields(void)
 struct vector *
 overview_extra_fields(void)
 {
-    struct vector *list;
+    struct vector *list = NULL;
+    struct vector *result = NULL;
     char *schema = NULL;
     char *line, *p;
     QIOSTATE *qp = NULL;
@@ -67,7 +68,7 @@ overview_extra_fields(void)
     qp = QIOopen(schema);
     if (qp == NULL) {
         syswarn("cannot open %s", schema);
-        goto fail;
+        goto done;
     }
     list = vector_new();
     for (field = 0, line = QIOread(qp); line != NULL; line = QIOread(qp)) {
@@ -104,14 +105,16 @@ overview_extra_fields(void)
             syswarn("error while reading %s", schema);
         }
     }
-    return list;
+    result = list;
 
- fail:
+done:
     if (schema != NULL)
         free(schema);
     if (qp != NULL)
         QIOclose(qp);
-    return NULL;
+    if (result == NULL && list != NULL)
+        vector_free(list);
+    return result;
 }
 
 
@@ -136,6 +139,24 @@ build_header(const char *article, size_t length, const char *header,
     if (end == NULL)
         return;
 
+    /* Someone managed to break their server so that they were appending
+       multiple Xref headers, and INN had a bug where it wouldn't notice this
+       and reject the article.  Just in case, see if there are multiple Xref
+       headers and use the last one. */
+    if (strcasecmp(header, "xref") == 0) {
+        const char *next = end + 1;
+
+        while (next != NULL) {
+            next = wire_findheader(next, length - (next - article), header);
+            if (next != NULL) {
+                data = next;
+                end = wire_endheader(data, article + length - 1);
+                if (end == NULL)
+                    return;
+            }
+        }
+    }
+
     size = end - data + 1;
     offset = overview->used + overview->left;
     buffer_resize(overview, offset + size);
@@ -174,7 +195,7 @@ overview_build(ARTNUM number, const char *article, size_t length,
     for (field = 0; field < ARRAY_SIZE(fields); field++) {
         buffer_append(overview, "\t", 1);
         if (field == 5) {
-            snprintf(buffer, sizeof(buffer), "%u", length);
+            snprintf(buffer, sizeof(buffer), "%lu", (unsigned long) length);
             buffer_append(overview, buffer, strlen(buffer));
         } else
             build_header(article, length, fields[field], overview);
