@@ -1,4 +1,4 @@
-/*  $Id: timecaf.c,v 1.27 2003/01/19 01:15:40 rra Exp $
+/*  $Id: timecaf.c 7412 2005-10-09 03:44:35Z eagle $
 **
 **  Like the timehash storage method (and heavily inspired by it), but uses
 **  the CAF library to store multiple articles in a single file.
@@ -468,6 +468,10 @@ static ARTHANDLE *OpenArticle(const char *path, ARTNUM artnum, const RETRTYPE am
 	    return NULL;
 	}
 	mmap_invalidate(private->mmapbase, private->mmaplen);
+        if (amount == RETR_ALL)
+            madvise(private->mmapbase, private->mmaplen, MADV_WILLNEED);
+        else
+            madvise(private->mmapbase, private->mmaplen, MADV_SEQUENTIAL);
 	private->artdata = private->mmapbase + delta;
     } else {
         private->artdata = xmalloc(private->artlen);
