@@ -1,4 +1,4 @@
-/*  $Id: tradspool.c,v 1.50.2.2 2004/12/19 08:19:48 rra Exp $
+/*  $Id: tradspool.c 7412 2005-10-09 03:44:35Z eagle $
 **
 **  Storage manager module for traditional spool format.
 */
@@ -776,6 +776,11 @@ OpenArticle(const char *path, RETRTYPE amount) {
 	    close(fd);
 	    return NULL;
 	}
+        if (amount == RETR_ALL)
+            madvise(private->artbase, sb.st_size, MADV_WILLNEED);
+        else
+            madvise(private->artbase, sb.st_size, MADV_SEQUENTIAL);
+
 	/* consider coexisting both wireformatted and nonwireformatted */
 	p = memchr(private->artbase, '\n', private->artlen);
 	if (p == NULL || p == private->artbase) {
